﻿#ifndef CFP_ARRAY_H_
#define CFP_ARRAY_H_

#include "CfpTree.h"
#include "../FimLogger/Logger.h"
#include "DatasetPreprocessor.h"

#include <string>
using std::string;
#include <stack>
using std::stack;

struct ElementInfo
{
	unsigned char* node;
	unsigned int deltaItem;
	unsigned int item;
	unsigned int count;
	bool visited;
	int numChildren;
};

class CfpArray
{
private:
	LogData* logData;
	CfpTree* cfpTree;
	int subarrayCount;
	unsigned long long arraySize;
	unsigned char* cfpArray;
	unsigned long long* subarrayStartIndices;
	int* remapToInitialId;
	int transactionCount;
	double minSup;

	stack<ElementInfo>* dfsStack;
	stack<unsigned int>* dfsCountStack;

	unsigned long long writeElementToArray(unsigned int byteIndex,
										   unsigned int deltaItem,
										   long long deltaPos,
										   unsigned int count);

	unsigned long long getCfpArraySize();
	unsigned long long getCfpArraySize(unsigned char* startNode);
	void transformTreeToArray();
	int pushAllSiblingsOnStack(unsigned char* node, unsigned int parentItem, stack<ElementInfo>* siblings);
	void performDfsIteration(ElementInfo& currentNode, stack<ElementInfo>& nodes);

	void printArray();
public:
	CfpArray(LogData* logData, DatasetPreprocessor* datasetPreprocessor);
	~CfpArray();

	void buildCfpArray(CfpTree* cfpTree);

	unsigned long long readElementFromArray(unsigned long long byteIndex,
											unsigned int& deltaItem,
											long long& deltaPos,
											unsigned int* count);

	unsigned long long getArraySize();
	int getSubarrayCount();
	unsigned long long* getSubarrayStartIndices();
	int* getRemapToInitialIdArray();
	double getMinimumSupport();
	int getTransactionCount();
};

#endif // CFP_ARRAY_H_
