﻿#include "CfpGrowth.h"
#include "FileLoader.h"
#include "CfpTree.h"
#include "CfpArray.h"
#include "Miner.h"
#include "CfpMiner.h"
#include "ShaFemMiner.h"
#include "FPGrowthMiner.h"
#include "DatasetPreprocessor.h"

#include <cstdlib>

CfpGrowth::CfpGrowth(InputData& inputData, LogData& logData)
	: inputData(inputData), logData(logData)
{
}

void CfpGrowth::startAlgorithm()
{
	int oldNumThreads = inputData.numThreads;
	int numThreads = 1; // TODO: change this when multithreaded CfpTree is implemented

	DatasetPreprocessor* datasetPreprocessor = NULL;
	CfpTree* cfpTree = NULL;
	CfpArray* cfpArray = NULL;
	Miner* miner = NULL;

	MultiBuffer** multiBuffers = new MultiBuffer*[numThreads];
	FileLoader::loadFileToMultiBuffers(inputData.datasetPath, multiBuffers, numThreads);

	datasetPreprocessor = new DatasetPreprocessor(multiBuffers, inputData.minSup, numThreads);

	cfpTree = new CfpTree(multiBuffers, datasetPreprocessor, &logData);
	cfpTree->buildTree();

	cfpArray = new CfpArray(&logData, datasetPreprocessor);
	cfpArray->buildCfpArray(cfpTree);
	delete cfpTree;

	if (inputData.miningMethod == MiningMethod::CFP_MINER)
		miner = new CfpMiner(cfpArray, oldNumThreads, &logData);
	else if (inputData.miningMethod == MiningMethod::SHAFEM_MINER)
		miner = new ShaFemMiner(cfpArray, oldNumThreads, &logData);
	else if (inputData.miningMethod == MiningMethod::FP_GROWTH_MINER)
		miner = new FPGrowthMiner(cfpArray, oldNumThreads, &logData);
	miner->startMining();
	if (inputData.writeOutput)
		miner->writeResults(algorithmName + "_output.txt");
	else
		miner->reportNumberOfFrequentItemsetsFound();
	delete miner;

	delete cfpArray;
	delete datasetPreprocessor;
}
