#ifndef CFP_TREE_MEMORY_MANAGER_H_
#define CFP_TREE_MEMORY_MANAGER_H_

#include <stack>
using std::stack;

const int SMALLEST_CFP_NODE_IN_BYTES = 7;
const int LARGEST_CFP_NODE_IN_BYTES = 24;
const long long MEMORY_BUFFER_SIZE = 1024 * 1024;
// This depends on the available memory in the system.
const int BUFFER_COUNT = ((long long)5 * 1024 * 1024 * 1024) / MEMORY_BUFFER_SIZE;

class CfpTreeMemoryManager
{
private:
	unsigned char** memoryBuffers;
	int bufferCount;
	int bufferIndex;
	unsigned long long totalMemoryUsage; // In bytes

	/*
	Multiple stacks containing free memory for each node size.
	*/
	stack<unsigned char*>** freeMemoryStack;

	void initializeStacks();
	unsigned char* allocateMemoryFromStack(int byteCount);
	void addFreeMemoryToStack(unsigned char* pointer, int byteCount);
	void allocateMemoryBuffer(const unsigned int bufferNr);
public:
	CfpTreeMemoryManager();
	~CfpTreeMemoryManager();
	unsigned char* allocateMemory(int byteCount);
	void freeMemory(unsigned char* pointer, int byteCount);
	unsigned long long getMemoryUsageInBytes();
};

#endif // CFP_TREE_MEMORY_MANAGER_H_
