#ifndef CFP_TREE_UTILITY_H_
#define CFP_TREE_UTILITY_H_

#include "CfpTreeMemoryManager.h"

#include <cstdlib>

const int POINTER_SIZE_IN_BYTES = 5;

struct UncompressedNode
{
	unsigned int deltaItem = 0;
	unsigned int pcount = 0;
	unsigned char* left = NULL;
	unsigned char* suffix = NULL;
	unsigned char* right = NULL;
};

struct PointedByInfo
{
	unsigned char* pointedBy = NULL;
	bool wasPointedByLeft = false;
	bool wasPointedBySuffix = false;
	bool wasPointedByRight = false;
};

void readCfpNode(const unsigned char* node,
				 unsigned int* deltaItem,
				 unsigned int* pcount,
				 unsigned char** left,
				 unsigned char** suffix,
				 unsigned char** right);
unsigned int getByteCountOfCfpNode(unsigned char bitmask);
unsigned int getDeltaItemSize(unsigned char bitmask);
unsigned int getPcountSize(unsigned char bitmask);
bool isLeftPointerNull(unsigned char bitmask);
bool isSuffixPointerNull(unsigned char bitmask);
bool isRightPointerNull(unsigned char bitmask);
bool areAllPointersNull(unsigned char bitmask);
bool isChainNode(unsigned char bitmask);
unsigned char* readChainNodePointer(const unsigned char* chainNode);
unsigned int readChainNodeItemDelta(const unsigned char* chainNode, int index);
unsigned int getNumNodesInChain(unsigned char bitmask);
void readEmbeddedLeafNode(const unsigned char*, unsigned int*, unsigned int*);
bool isPointerEmbeddedLeafNode(unsigned char* pointer);
void decompressCfpNode(unsigned char* node, UncompressedNode* unode);

unsigned char* createCfpNode(UncompressedNode* unode, CfpTreeMemoryManager* memoryManager);
void editNode(unsigned char*,
			  unsigned int*,
			  unsigned int*,
			  unsigned char**,
			  unsigned char**,
			  unsigned char**);
unsigned char* createChainNodes(unsigned int* items,
								unsigned int itemCount,
								unsigned int itemIndex,
								unsigned int delta,
								CfpTreeMemoryManager *memoryManager);
void addNodesToLastSuffixes(const int,
							unsigned char**,
							int*,
							unsigned int*,
							UncompressedNode&,
							CfpTreeMemoryManager *memoryManager);
unsigned char* createChainNodeWithNullPointer(unsigned int* deltaItems,
											  unsigned int itemCount,
											  CfpTreeMemoryManager *memoryManager);
void editPointerInChainNode(unsigned char* chainNode, unsigned char* pointer);
unsigned char* splitChainNode(unsigned char* chainNode,
							  unsigned int chainSplitIndex,
							  unsigned int *items,
							  unsigned int itemCount,
							  unsigned int delta,
							  bool isLeftPointer,
							  CfpTreeMemoryManager *memoryManager);
void expandNode(UncompressedNode* unode,
				unsigned char* oldNode,
				PointedByInfo pointedByInfo,
				CfpTreeMemoryManager* memoryManager,
				unsigned char **rootNode);
void updatePointedByPointer(unsigned char* newNode,
							PointedByInfo pointedByInfo,
							unsigned char **rootNode);
void createEmbeddedLeafNode(UncompressedNode* unode, unsigned char* node, int pointerByteIndex);

#endif // CFP_TREE_UTILITY_H_
