#ifndef FP_TREE_H_
#define FP_TREE_H_

#include "StackMemoryManager.h"
#include "../FimLogger/Logger.h"
#include "../ShaFEM/HashTable.h"

struct FPNode
{
	FPNode* parent;
	FPNode* firstChild;
	FPNode* nextSibling;
	FPNode* nextEqualItem;
	int item;
	int itemCount;
};

struct HeaderTable
{
	int* itemCounts; // The position of the element represents the item ID.
	int numItems;
	FPNode** nodes;
};

struct HashHeaderTableElement
{
	FPNode* node;
	int itemCount;
};

class FPTree
{
	private:
		StackMemoryManager* memoryManager;
		bool memoryManagerFromOutside;
		HashTable<HashHeaderTableElement*>* hashHeaderTable; // Used in conditional FP-trees
		HeaderTable headerTable;
		FPNode* rootNode;

		double minSup;
		int numTransactions;
		int longestTransaction;

		void setRootNode(FPNode* root);
		FPNode* allocateNode();
		void printFPTree(FPNode* node);
	public:
		FPTree();
		FPTree(StackMemoryManager* stackMemoryManager);
		~FPTree();

		void setHashHeaderTable(HashTable<HashHeaderTableElement*>* table);
		HashTable<HashHeaderTableElement*>* getHashHeaderTable();
		FPNode* getRootNode();
		HeaderTable* getHeaderTable();
		unsigned long long getMemoryUsage();

		bool containsSinglePathOnly(int item);
		void insertTransaction(int* transaction, int itemCount, int supportCount = -1);
};

#endif // FP_TREE_H_
