#include "ItemsetStack.h"

ItemsetStack::ItemsetStack(StackMemoryManager &memoryManager)
	: memoryManager(memoryManager)
{
	itemsetCount = 0;
	ownMemoryManager = false;
}

ItemsetStack::ItemsetStack()
	: memoryManager(*(new StackMemoryManager()))
{
	itemsetCount = 0;
	ownMemoryManager = true;
}

ItemsetStack::~ItemsetStack()
{
	if (ownMemoryManager)
		delete &memoryManager;
}

/***** PUBLIC FUNCTIONS *****/

unsigned int* ItemsetStack::push(unsigned int itemCount, double support)
{
	int sizeofItems = itemCount * sizeof(unsigned int);
	int sizeofSupport = sizeof(double);
	int sizeofItemCount = sizeof(unsigned int);

	unsigned int memoryChunkSize = sizeofItems + sizeofSupport + sizeofItemCount;;
	unsigned char* itemAddress = memoryManager.grow(memoryChunkSize);

	double* supportAddress = (double*)(itemAddress + sizeofItems);
	*supportAddress = support;

	topItemCountAddress = (unsigned int*)(itemAddress + sizeofItems + sizeofSupport);
	*topItemCountAddress = itemCount;

	itemsetCount++;

	return (unsigned int*)itemAddress;
}

void ItemsetStack::top(unsigned int** itemCount, double** support, unsigned int** items)
{
	*itemCount = topItemCountAddress;
	*support = (double*)((unsigned char*)topItemCountAddress - sizeof(double));
	*items = (unsigned int*)((unsigned char*)topItemCountAddress - sizeof(double) - (**itemCount) * sizeof(unsigned int));
}

void ItemsetStack::pop()
{
	unsigned int itemCount = *topItemCountAddress;
	long long shrinkAmount = itemCount * sizeof(unsigned int) + sizeof(double) + sizeof(unsigned int);
	memoryManager.shrink(shrinkAmount);
	topItemCountAddress = (unsigned int*)((unsigned char*)topItemCountAddress - shrinkAmount);

	itemsetCount--;
}

int ItemsetStack::size()
{
	return itemsetCount;
}

/***** PRIVATE FUNCTIONS *****/
