#include "Miner.h"

#include <cmath>
#include <iostream>
#include <fstream>
using std::cout;
using std::endl;
using std::ofstream;
#include <omp.h>

Miner::Miner(CfpArray* cfpArray, int numThreads, LogData* logData)
	: cfpArray(*cfpArray), numThreads(numThreads), logData(logData)
{
	itemsetStacks = new ItemsetStack*[numThreads];
	for (int i = 0; i < numThreads; i++)
		itemsetStacks[i] = new ItemsetStack;

	minSup = cfpArray->getMinimumSupport();
	numTransactions = cfpArray->getTransactionCount();
	minNumTransactions = (int)ceil(minSup * numTransactions);
	if (minNumTransactions == 0)
		minNumTransactions = 1; // An itemset must occur in at least one transaction
}

Miner::~Miner()
{
	for (int i = 0; i < numThreads; i++)
		delete itemsetStacks[i];
	delete[] itemsetStacks;
}

void Miner::writeResults(string path)
{
	reportNumberOfFrequentItemsetsFound();
	string startMessage = "Writing file to: " + path + "\n";
	writeToLog(startMessage);
	double time = omp_get_wtime();

	ofstream file;
	file.open(path);

	for (int k = 0; k < numThreads; k++)
	{
		for (int i = (int)itemsetStacks[k]->size() - 1; i >= 0; i--)
		{
			unsigned int* itemCount;
			double* support;
			unsigned int* items;
			itemsetStacks[k]->top(&itemCount, &support, &items);
			for (unsigned int j = 0; j < *itemCount; j++)
				file << items[j] << " ";
			file << "(" << *support << ")" << std::endl;
			itemsetStacks[k]->pop();
		}
	}

	file.close();

	string endMessage = "Finished writing. [" + toString(omp_get_wtime() - time) + "s]\n\n";
	writeToLog(endMessage);

	std::ifstream in(path, std::ios::binary | std::ios::ate);
	writeToLog("Created file size: " + std::to_string((int)ceil(in.tellg() / 1024.0)) + "kB\n\n");
}

void Miner::reportNumberOfFrequentItemsetsFound()
{
	int numFrequentItemsets = 0;
	for (int i = 0; i < numThreads; i++)
		numFrequentItemsets += (int)itemsetStacks[i]->size();
	writeToLog("Number of frequent itemsets found: " + std::to_string(numFrequentItemsets) + "\n");
}

bool Miner::addToFrequentItemsets(int threadNr, int* items, unsigned int itemCount, int supportCount)
{
	if (itemsetStacks[threadNr]->size() > 500000000)
	{
		writeToLog("Too many itemsets found! Stopping  mine procedure.");
		return true;
	}

	double support = 100 * supportCount / (double)numTransactions;
	unsigned int* itemAddress = itemsetStacks[threadNr]->push(itemCount, support);
	for (int i = 0; i < itemCount; i++)
		itemAddress[i] = cfpArray.getRemapToInitialIdArray()[items[i]];

	return false;
}
