#ifndef SHA_FEM_MINER_H_
#define SHA_FEM_MINER_H_

#include "Miner.h"
#include "StackMemoryManager.h"
#include "FPTree.h"
#include "BitsetUtility.h"
#include "HashTable.h"
#include "CustomVector.h"

struct BitsetVector
{
	Bitset* bitsets;
	int itemCount;
	int* items;
};

struct CalculateSupportCountContext
{
	// For MineBitVector
	BitsetVector* bitsetVector;
	CustomVector<Bitset>* bitsets;
	int* weights;
	int itemCount;
	unsigned int bitCounter;

	// For MineFPTree
	HashTable<HashHeaderTableElement*>* hashHeaderTable;
	int currentItem;
};

class ShaFemMiner : public Miner
{
private:
	StackMemoryManager** memoryManagers;
	const int STEP;
	const int N;
	int* K;
	int** X;
	double timeBitVectorMining;

	void ParallelMinePattern();
	void MineFPTree(FPTree* tree, int* suffix, int suffixSize);
	void StartMineBitVector(FPNode* node, int sizeOfConditionalPatternBase);
	void MineBitVector(BitsetVector* bitsetVector, int* weights, int* suffix, int suffixSize);
	void UpdateK(int numNewPatters, int size, int threadNr);

	int getSizeOfConditionalPatternBase(FPNode* bottomNode, int* numUniqueItems);
	void joinTwoArrays(int* array1, int sizeArray1, int* array2, int sizeArray2, int* destination);
	int getSupport(Bitset* bitset, int* weights);
	void putAllItemsFromConditionalPatternBaseToSet(HashTable<int>* set, FPNode* node);
	void initializeBitsets(BitsetVector* bitsetVector, FPNode* node);
	void setWeights(int* weights, FPNode* node);
	void constructConditionalFpTree(FPNode* firstNode, FPTree* fpTree);
	int constructConditionalFpTree(unsigned int initialItem, FPTree* fpTree, int *numUniqueItems);

	void addRestOfItemsToFrequentItemsets(CustomVector<int>* items,
										  int* currentItemset,
										  int currentItemsetSize,
										  int (ShaFemMiner::*calcSupport)(CalculateSupportCountContext*),
										  CalculateSupportCountContext* context);
	int calcuateSupportForBitsetVector(CalculateSupportCountContext* context);
	int calculateSupportForFpTree(CalculateSupportCountContext* context);
	int calculateSupportForFPTreeNonSinglePath(FPNode* bottomNode, int forItem);
public:
	ShaFemMiner(CfpArray* cfpArray, int numThreads, LogData* logData);
	~ShaFemMiner();

	void startMining();
};

#endif // SHA_FEM_MINER_H_
