#ifndef DATASET_GENERATOR_
#define DATASET_GENERATOR_

#include <string>
using std::string;
#include <vector>
using std::vector;
#include <random>

struct ExtendedItem
{
	int item;
	int dat;

	bool operator==(const ExtendedItem& extItem) const
	{
		return item == extItem.item && dat == extItem.dat;
	}
};

struct InputData
{
	int numTransactions = 1000;
	int averageTransactionLength = 30;
	int numUniqueItems = 400;
	int numThreads = 1;
	int maxspan = 6;
};

class DatasetGenerator
{
private:
	InputData& inputData;
	int averageSizeOfMegatransactions;
	int numberOfMegatransactions;
	double poissonDistrMeanForMegatransactions;
	double poissonDistrMeanForTransactions;
	double expDistrRateForMegatransactions;
	double expDistrRateForTransactions;
	double normalDistrMean;
	double normalDistrVariance;

	std::default_random_engine** generators;
	vector<vector<ExtendedItem>* > megatransactions;
	double* megatransactionWeights;

	int* itemCount;

	int** transactions;
	int* transactionLengths;
	int* items;

	void createMegatransactions();
	void createMegatransactionWeights();
	void setTransactionLengths();
	void setAndRandomizeItems();
	void assignMegatransactionsToTransactions();
	void assignNonAssignedItems();
	int getNextMegatransactionIndex();
	int getRandomInt(int min, int max);
public:
	DatasetGenerator(InputData& inputData);
	~DatasetGenerator();

	void writeToPath(string path);
};

#endif // DATASET_GENERATOR
