#include "Logger.h"

#include <iostream>
using std::cout;
using std::endl;
#include <fstream>
using std::ofstream;
using std::ios;
#include <sstream>
#include <iterator>
#include <cstdlib>
#include <istream>
using std::ifstream;

string toString(double value)
{
	std::ostringstream ss;
	ss << value;
	return ss.str();
}

void writeToLog(string message)
{
	cout << message;
	ofstream file;
	file.open(algorithmName + "_log.txt", ios::app);
	file << message;
	file.close();
}

// From http://stackoverflow.com/questions/997946/how-to-get-current-time-and-date-in-c
const std::string currentDateTime() {
	time_t     now = time(0);
	struct tm  tstruct;
	char       buf[80];
	tstruct = *localtime(&now);
	// Visit http://en.cppreference.com/w/cpp/chrono/c/strftime
	// for more information about date/time format
	strftime(buf, sizeof(buf), "%Y.%m.%d %X", &tstruct);

	return buf;
}

Logger::Logger(string inputFilePath, string logFilePath)
	: logFilePath(logFilePath)
{
	if (!readInputFile(inputFilePath))
	{
		writeToLog("Error reading input file.\n");
		exit(EXIT_FAILURE);
	}

	ifstream f(logFilePath);
	if (!f)
	{ // If file does not exist, create a new one.
		ofstream file;
		file.open(logFilePath);
		string message = "";
		message += "algorithmName ";
		message += "repetitionNr ";
		message += "support ";
		message += "threads ";
		message += "dataset ";
		message += "maxspan ";
		message += "preprocessTime ";
		message += "mineTime ";
		message += "memoryUsageInMB ";
		file << message << endl;
		file.close();
	}
	f.close();
}

vector<int>* Logger::getThreads()
{
	return &threads;
}

vector<double>* Logger::getSupports()
{
	return &supports;
}

vector<string>* Logger::getDatasets()
{
	return &datasets;
}

vector<int>* Logger::getMaxspans()
{
	return &maxspans;
}

int Logger::getNumRepetitions() const
{
	return numRepetitions;
}

void Logger::writeLogData(LogData& logData)
{
	ofstream file;
	file.open(logFilePath.c_str(), ios::app);
	string data = "";
	data += logData.algorithmName + " ";
	data += toString(logData.repetitionNr) + " ";
	data += toString(supports[logData.supportIndex]) + " ";
	data += toString(threads[logData.threadIndex]) + " ";
	data += datasets[logData.datasetIndex] + " ";
	data += toString(maxspans[logData.maxspanIndex]) + " ";
	data += toString(logData.preprocessTime) + " ";
	data += toString(logData.mineTime) + " ";
	data += toString(logData.memoryUsageInMB) + " ";
	file << data << endl;
	file.close();
}

void Logger::writeLogData(LogData& logData, string logFilePath)
{
	ifstream f(logFilePath);
	if (!f)
	{ // If file does not exist, create a new one.
		ofstream file;
		file.open(logFilePath);
		string message = "";
		message += "algorithmName ";
		message += "repetitionNr ";
		message += "support ";
		message += "threads ";
		message += "dataset ";
		message += "maxspan ";
		message += "preprocessTime ";
		message += "mineTime ";
		message += "memoryUsageInMB ";
		file << message << endl;
		file.close();
	}
	f.close();

	ofstream file;
	file.open(logFilePath.c_str(), ios::app);
	string data = "";
	data += logData.algorithmName + " ";
	data += toString(logData.repetitionNr) + " ";
	data += toString(logData.support) + " ";
	data += toString(logData.threads) + " ";
	data += logData.dataset + " ";
	data += toString(logData.maxspan) + " ";
	data += toString(logData.preprocessTime) + " ";
	data += toString(logData.mineTime) + " ";
	data += toString(logData.memoryUsageInMB) + " ";
	file << data << endl;
	file.close();
}

/*
Input file format:

THREADS: 1 2 3 4 5
SUPPORTS: 0.1 0.2 0.3
DATASETS: retail.dat kosarak.dat
MAXSPANS: 0 1 2 3
REPETITIONS: 3
*/
bool Logger::readInputFile(string path)
{
	std::ifstream file;
	file.open(path.c_str());

	int lineNr = 0;
	while (!file.eof())
	{
		string s;
		std::getline(file, s);
		std::stringstream ss(s);
		std::istream_iterator<std::string> begin(ss);
		std::istream_iterator<std::string> end;
		std::vector<std::string> vstrings(begin, end);

		for (int i = 1; i < vstrings.size(); i++)
		{
			if (lineNr == 0)
			{ // THREADS
				threads.push_back(std::atoi(vstrings[i].c_str()));
			}
			else if (lineNr == 1)
			{ // SUPPORTS
				supports.push_back(std::atof(vstrings[i].c_str()));
			}
			else if (lineNr == 2)
			{ // DATASETS
				datasets.push_back(vstrings[i]);
			}
			else if (lineNr == 3)
			{ // MAXSPANS
				maxspans.push_back(std::atoi(vstrings[i].c_str()));
			}
			else if (lineNr == 4)
			{ // REPETITIONS
				numRepetitions = std::atoi(vstrings[i].c_str());
			}
		}
		lineNr++;

		if (lineNr >= 5)
			break;
	}

	return true;
}
