#ifndef LOGGER_H_
#define LOGGER_H_

#include <string>
using std::string;
#include <vector>
using std::vector;

extern string algorithmName;

string toString(double value);
void writeToLog(string message);
const string currentDateTime();

struct LogData
{
	int repetitionNr;
	int supportIndex;
	int threadIndex;
	int datasetIndex;
	int maxspanIndex;
	double preprocessTime;
	double mineTime;
	double memoryUsageInMB;

	double support;
	int threads;
	string dataset;
	string algorithmName;
	int maxspan;

	LogData()
	{
		repetitionNr = 0;
		supportIndex = 0;
		threadIndex = 0;
		datasetIndex = 0;
		maxspanIndex = 0;
		preprocessTime = 0.0;
		mineTime = 0.0;
		memoryUsageInMB = 0.0;

		support = 0.0;
		threads = 0;
		dataset = "";
		algorithmName = "";
		maxspan = 0;
	}
};

class Logger
{
private:
	string logFilePath;
	vector<int> threads;
	vector<double> supports;
	vector<string> datasets;
	vector<int> maxspans;
	int numRepetitions;

	bool readInputFile(string path);
public:
	Logger(string inputFilePath, string logFilePath);
	void writeLogData(LogData& logData);
	static void writeLogData(LogData&, string logFilePath);

	vector<int>* getThreads();
	vector<double>* getSupports();
	vector<string>* getDatasets();
	vector<int>* getMaxspans();
	int getNumRepetitions() const;
};

#endif /* LOGGER_H_ */
