#ifndef BIT_TABLE_H_
#define BIT_TABLE_H_

#include "StackMemoryManager.h"
#include "MultiBuffer.h"
#include "Utility.h"
#include "DatasetPreprocessor.h"
#include "../FimLogger/Logger.h"

class BitTable
{
private:
	StackMemoryManager** memoryManagers;
	MultiBuffer** multiBuffers;
	InputData& inputData;
	DatasetPreprocessor& datasetPreprocessor;
	LogData& logData;
	int numRowsForItemTable;
	int numColumnsForItemTable;
	int numBytesPerRowForItemTable;
	int numRowsForTransactionTable;
	int numColumnsForTransactionTable;
	int numBytesPerRowForTransactionTable;
	unsigned char** itemTable;
	unsigned char** transactionTable;

	void allocateMemory();
	void createBitTable();
	void setBitsToTrue(int item, int transactionNr);
	int getFirstTransactionNr(int threadNr) const;
	int getLastTransactionNr(int threadNr) const;
public:
	BitTable(MultiBuffer** multiBuffers,
			 InputData& inputData,
			 DatasetPreprocessor& datasetPreprocessor,
			 StackMemoryManager** memoryManagers,
			 LogData &logData);
	~BitTable();

	int getNumRowsForItemTable() const;
	int getNumColumnsForItemTable() const;
	int getNumBytesPerRowForItemTable() const;
	int getNumRowsForTransactionTable() const;
	int getNumColumnsForTransactionTable() const;
	int getNumBytesPerRowForTransactionTable() const;

	void itemIdToExtendedItem(unsigned int itemId,
							  unsigned int *itemOut,
							  unsigned char *dimensionalAttributeOut) const;
	void extendedItemToItemId(unsigned int item,
							  unsigned char dimensionalAttribute,
							  unsigned int* itemIdOut) const;
	void getItemsFromSubsumbeBitVector(unsigned char* subsumeBitVector,
									   unsigned int* subsumeItemsOut,
									   unsigned int itemToIgnore) const;
	bool isItemInTransaction(int row, int column) const;
	unsigned char* getItemBitVector(unsigned int item) const;
	unsigned char* getTransactionBitVector(unsigned int transactionNr) const;
	unsigned long long getMemoryUsageInBytes() const;
};

#endif // BIT_TABLE_H_
