#include "BitsetUtility.h"

#include <cmath>
using std::pow;
#include <iostream>
using std::cout;
using std::endl;
#include <bitset>

void printBitset(unsigned char* bitset, int byteCount)
{
	for (int i = 0; i < byteCount; i++)
	{
		std::bitset<8> byte(bitset[i]);
		cout << byte << " ";
	}
	cout << endl;
}

void setBytesInBitset(unsigned char* bitset, int byteIndex, int byteCount, const void* bytes)
{
	for (int i = byteCount - 1; i >= 0; i--)
		bitset[byteIndex + (byteCount - i - 1)] = (unsigned char)(*((unsigned long long*)bytes) >> (i * 8));
}

unsigned long long getValueFromBitset(const unsigned char* bitset, int byteIndex, int byteCount)
{
	unsigned long long value = 0;
	for (int i = byteCount - 1; i >= 0; i--)
		value = (value << 8) | bitset[byteIndex + (byteCount - i - 1)];
	return value;
}

bool getBitAt(const unsigned char* bitset, const int bitIndex)
{
	int byteIndex = bitIndex / 8;
	int localIndex = bitIndex - (bitIndex / 8) * 8;
	return (bitset[byteIndex] & (0x80 >> localIndex)) != 0;
}

void setBitAt(unsigned char* bitset, int bitIndex, bool value)
{
	int byteIndex = bitIndex / 8;
	int localIndex = bitIndex - (bitIndex / 8) * 8;

	if (value)
		bitset[byteIndex] |= (0x80 >> localIndex);
	else
		bitset[byteIndex] &= ~(0x80 >> localIndex);
}

unsigned long long getNumBytes(unsigned long long numBits)
{
	return ceil(numBits / 8.0);
}

/*
Sets all bytes to zero.
*/
void zeroBytes(unsigned char* bytes, int numBytes)
{
	for (int i = 0; i < numBytes; i++)
		bytes[i] = 0;
}

void intersectBitsets(const unsigned char* bitset1,
					  const unsigned char* bitset2,
					  const unsigned long long byteCount,
					  unsigned char* result)
{
	unsigned long long num4Bytes = byteCount / 4;
	unsigned long long i;
	for (i = 0; i < num4Bytes; i++)
		((unsigned int*)result)[i] = ((unsigned int*)bitset1)[i] & ((unsigned int*)bitset2)[i];
	i *= 4;
	for (; i < byteCount; i++)
		result[i] = bitset1[i] & bitset2[i];
}

unsigned long long popcount(const unsigned char* bitset,
							const long long bitCount)
{
	const long long numWholeBytes = bitCount / 8;
	unsigned long long count = 0;

	long long i = 0;
	for (; i < numWholeBytes - 4; i += 4)
	{
		count += __builtin_popcount(*(unsigned int*)(bitset + i));
	}
	i = i * 8;
	for (; i < bitCount; i++)
		count += getBitAt(bitset, i) ? 1 : 0;

	return count;
}
