#ifndef DATASET_PREPROCESSOR_H_
#define DATASET_PREPROCESSOR_H_

#include "MultiBuffer.h"
#include "Utility.h"

int compareItem(const void* itemIdA, const void* itemIdB);

class DatasetPreprocessor
{
private:
	MultiBuffer** multiBuffers;
	InputData& inputData;
	int supportBufferSize = 0;
	int longestTransaction = 0;
	int transactionCount = 0;
	int* numTransactionsForThread;
	int numTransactionsWithFrequentItems = 0;
	int numFrequentItems = 0;
	int* initialToRemapId;
	int* remapToInitialId;

	void countSupport();
	void getMaxIdAndLongestTransaction(int* maxId, int* longestTransaction);
	void countTransactionsWithFrequentItems();
	void countNumberOfFrequentItems();
	void remapItemsByFrequency();
public:
	static int* supportCount;

	DatasetPreprocessor(MultiBuffer** multiBuffers, InputData& inputData);
	~DatasetPreprocessor();

	int getNumTransactions() const;
	int getSupportBufferSize() const;
	int getSupportCount(unsigned int item, unsigned int dimensionalAttribute) const;
	bool isItemFrequent(unsigned int extendedItemIndex) const;
	int getMinNumTransactions() const;
	int getNumTransactionsForThread(int threadNr) const;
	int getNumTransactionsWithFrequentItems() const;
	int getNumFrequentItems() const;
	int* getRemapToInitialIdArray() const;
	int* getInitialToRemapIdArray() const;
};

#endif // DATASET_PREPROCESSOR_H_
