#include "ExtendedItemsetStack.h"

#include <cstdlib>

ExtendedItemsetStack::ExtendedItemsetStack()
{
	itemsetCount = 0;
	topItemCountAddress = NULL;
}

/***** PUBLIC FUNCTIONS *****/

/*
 * Order: items, dimensional attributes, support, item count
 */
void ExtendedItemsetStack::push(unsigned int itemCount,
								double support,
								unsigned char** dimensionalAttributeAddressOut,
								unsigned int** itemAddressOut)
{
	unsigned long long sizeofItems = itemCount * sizeof(unsigned int);
	unsigned long long sizeofDimenstionalAttributes = itemCount * sizeof(unsigned char);
	unsigned long long sizeofSupport = sizeof(double);
	unsigned long long sizeofItemCount = sizeof(unsigned int);

	unsigned long long memoryChunkSize = sizeofItems + sizeofDimenstionalAttributes + sizeofSupport + sizeofItemCount;
	unsigned char* itemAddress = memoryManager.grow(memoryChunkSize);

	double* supportAddress = (double*)(itemAddress + sizeofItems + sizeofDimenstionalAttributes);
	*supportAddress = support;

	topItemCountAddress = (unsigned int*)(itemAddress + sizeofItems + sizeofDimenstionalAttributes + sizeofSupport);
	*topItemCountAddress = itemCount;

	itemsetCount++;

	*itemAddressOut = (unsigned int*)itemAddress;
	*dimensionalAttributeAddressOut = itemAddress + sizeofItems;
}

void ExtendedItemsetStack::top(unsigned int** itemCount,
							   double** support,
							   unsigned char** dimensionalAttributes,
							   unsigned int** items) const
{
	unsigned char* address = (unsigned char*)topItemCountAddress;
	*itemCount = topItemCountAddress;
	address -= sizeof(double);
	*support = (double*)address;
	address -= (**itemCount) * sizeof(unsigned char);
	*dimensionalAttributes = address;
	address -= (**itemCount) * sizeof(unsigned int);
	*items = (unsigned int*)address;
}

void ExtendedItemsetStack::pop()
{
	unsigned int itemCount = *topItemCountAddress;
	long long shrinkAmount = itemCount * (sizeof(unsigned int) + sizeof(unsigned char)) + sizeof(double) + sizeof(unsigned int);
	memoryManager.shrink(shrinkAmount);
	topItemCountAddress = (unsigned int*)((unsigned char*)topItemCountAddress - shrinkAmount);

	itemsetCount--;
}

int ExtendedItemsetStack::size() const
{
	return itemsetCount;
}

unsigned long long ExtendedItemsetStack::getMemoryUsageInBytes() const
{
	return memoryManager.getMemoryUsageInBytes();
}
