#include "FineGrainedParallelMiner.h"
#include "BitsetUtility.h"

#include <iostream>
using std::cout;
using std::endl;

struct ThreadArgs
{
	FineGrainedParallelMiner* object;
	pthread_mutex_t* localLock;
	int* index;
	int threadId;
};

static void* staticIndexBTFITIPerThread(void* object)
{
	return ((ThreadArgs*)object)->object->indexBTFITIPerThread(((ThreadArgs*)object)->threadId,
															   ((ThreadArgs*)object)->localLock,
															   ((ThreadArgs*)object)->index);
}

static void* staticDepthFirstPerThread(void* object)
{
	return ((DepthFirstArgs*)object)->object->depthFirstPerThread((DepthFirstArgs*)object);
}

struct SubsumeCombinationsArgs
{
	ExtendedItemset* tail;
	FineGrainedParallelMiner* object;
	double support;
	int start;
	int end;
	unsigned int* items;
	unsigned int itemCount;
	unsigned int* subsume;
	unsigned int subsumeCount;
	int threadNr;
	unsigned char* dimensionalAttributes;
};

static void* staticAddAllSubsumeSetsPerThread(void* object)
{
	return ((SubsumeCombinationsArgs*)object)->object->addAllSubsumeSetsPerThread(((SubsumeCombinationsArgs*)object)->start,
																				  ((SubsumeCombinationsArgs*)object)->end,
																				  ((SubsumeCombinationsArgs*)object)->items,
																				  ((SubsumeCombinationsArgs*)object)->itemCount,
																				  ((SubsumeCombinationsArgs*)object)->subsume,
																				  ((SubsumeCombinationsArgs*)object)->subsumeCount,
																				  ((SubsumeCombinationsArgs*)object)->dimensionalAttributes,
																				  ((SubsumeCombinationsArgs*)object)->support,
																				  ((SubsumeCombinationsArgs*)object)->threadNr);
}

static void* staticCombineItemsSubsumeAndTailPerThread(void* object)
{
	return ((SubsumeCombinationsArgs*)object)->object->combineItemsSubsumeAndTailPerThread(((SubsumeCombinationsArgs*)object)->start,
																						   ((SubsumeCombinationsArgs*)object)->end,
																						   ((SubsumeCombinationsArgs*)object)->items,
																						   ((SubsumeCombinationsArgs*)object)->itemCount,
																						   ((SubsumeCombinationsArgs*)object)->subsume,
																						   ((SubsumeCombinationsArgs*)object)->subsumeCount,
																						   ((SubsumeCombinationsArgs*)object)->dimensionalAttributes,
																						   ((SubsumeCombinationsArgs*)object)->tail,
																						   ((SubsumeCombinationsArgs*)object)->threadNr);
}

FineGrainedParallelMiner::FineGrainedParallelMiner(StackMemoryManager **memoryManagers,
												   DatasetPreprocessor &datasetPreprocessor,
												   BitTable &bitTable,
												   IndexArray &indexArray,
												   InputData &inputData,
												   LogData &logData)
	: Miner(memoryManagers, datasetPreprocessor, bitTable, indexArray, inputData, logData)
{
	threads = new Thread[inputData.numThreads];
	for (int i = 0; i < inputData.numThreads; i++)
	{
		threads[i].id = i;
		threads[i].thread = new pthread_t;
		threads[i].isWorking = true;
		threads[i].finishedMainWork = false;
	}
	newThreadsAvailable = true;
	everythingFinished = false;
	if (pthread_mutex_init(&lock, NULL) != 0)
	{
		writeToLog("Mutex init failed.\n");
	}
}

FineGrainedParallelMiner::~FineGrainedParallelMiner()
{
	for (int i = 0; i < inputData.numThreads; i++)
		delete threads[i].thread;
	delete[] threads;

	pthread_mutex_destroy(&lock);
}

void FineGrainedParallelMiner::indexBTFITI()
{
	int numThreads = getNumThreadsNeeded(inputData.numThreads, indexArray.getNumElements());

	for (int i = numThreads; i < inputData.numThreads; i++)
	{
		threads[i].isWorking = false;
		threads[i].finishedMainWork = true;
		availableThreads.push_back(&threads[i]);
	}
	newThreadsAvailable = availableThreads.size() > 0;

	ThreadArgs* threadArgs = new ThreadArgs[inputData.numThreads];
	pthread_mutex_t localLock;
	if (pthread_mutex_init(&localLock, NULL) != 0)
		writeToLog("Mutex init failed.\n");
	int index = 0;

	for (int i = 0; i < inputData.numThreads; i++)
	{
		if (i == inputData.numThreads - 1)
		{
			indexBTFITIPerThread(threads[i].id, &localLock, &index);
		}
		else
		{
			threadArgs[i].object = this;
			threadArgs[i].threadId = threads[i].id;
			threadArgs[i].localLock = &localLock;
			threadArgs[i].index = &index;
			int err = pthread_create(threads[i].thread, NULL, staticIndexBTFITIPerThread, &threadArgs[i]);
			if (err != 0)
				writeToLog("Can't create thread " + toString(threads[i].id) + "\n");
		}
	}

	for (int i = 0; i < inputData.numThreads; i++)
		pthread_join(*(threads[i].thread), NULL);

	pthread_mutex_destroy(&localLock);

	delete[] threadArgs;
}

void* FineGrainedParallelMiner::indexBTFITIPerThread(int threadNr, pthread_mutex_t* localLock, int* index)
{
	int numThreads = getNumThreadsNeeded(inputData.numThreads, indexArray.getNumElements());

	if (threadNr < numThreads)
	{
		IndexArrayElement* elements = indexArray.getElements();

		int i = 0;
		while (true)
		{
			pthread_mutex_lock(localLock);
			i = *index;
			*index += 1;
			pthread_mutex_unlock(localLock);

			if (i < indexArray.getNumElements())
				indexBTFITIForItem(i, elements, threadNr);
			else
				break;
		}

		pthread_mutex_lock(&lock);
		threads[threadNr].finishedMainWork = true;
		threads[threadNr].isWorking = false;
		availableThreads.push_back(&threads[threadNr]);
		checkIfAllThreadsFinished();
		newThreadsAvailable = true;
		pthread_mutex_unlock(&lock);
	}

	while (!everythingFinished)
	{
		if (threads[threadNr].isWorking)
		{
			threads[threadNr].work(threads[threadNr].args);
			threads[threadNr].isWorking = false;
			checkIfAllThreadsFinished();
		}
	}

	return NULL;
}

void FineGrainedParallelMiner::depthFirst(ExtendedItemset& itemset, ExtendedItemset& tail, int threadNr)
{
	if (tail.itemCount == 0)
		return;

	ExtendedItemset* newItemset = (ExtendedItemset*)memoryManagers[threadNr]->grow(sizeof(ExtendedItemset));
	ExtendedItemset* newTail = (ExtendedItemset*)memoryManagers[threadNr]->grow(sizeof(ExtendedItemset));
	unsigned int itemsetItemCount = itemset.itemCount + 1;
	unsigned int tailItemCount = tail.itemCount - 1;
	long long sizeofItems = itemsetItemCount * sizeof(unsigned int*);
	long long sizeofDats = itemsetItemCount * sizeof(unsigned char*);
	long long sizeOfTailItems = tailItemCount * sizeof(unsigned int*);
	long long sizeOfTailDats = tailItemCount * sizeof(unsigned char*);
	newItemset->itemCount = itemsetItemCount;
	newItemset->items = (unsigned int*)memoryManagers[threadNr]->grow(sizeofItems);
	newItemset->dimensionalAttributes = memoryManagers[threadNr]->grow(sizeofDats);
	std::copy(itemset.items, itemset.items + itemset.itemCount, newItemset->items);
	std::copy(itemset.dimensionalAttributes, itemset.dimensionalAttributes + itemset.itemCount, newItemset->dimensionalAttributes);

	newTail->itemCount = tailItemCount;
	newTail->items = (unsigned int*)memoryManagers[threadNr]->grow(sizeOfTailItems);
	newTail->dimensionalAttributes = memoryManagers[threadNr]->grow(sizeOfTailDats);

	DepthFirstArgs* depthFirstArgs = (DepthFirstArgs*)memoryManagers[threadNr]->grow(sizeof(DepthFirstArgs));
	pthread_mutex_t localLock;
	if (pthread_mutex_init(&localLock, NULL) != 0)
		writeToLog("Mutex init failed.\n");
	int index = 0;

	depthFirstArgs->itemset = &itemset;
	depthFirstArgs->tail = &tail;
	depthFirstArgs->newItemset = newItemset;
	depthFirstArgs->newTail = newTail;
	depthFirstArgs->index = &index;
	depthFirstArgs->localLock = &localLock;
	depthFirstArgs->object = this;
	depthFirstArgs->threadNr = threadNr;
	depthFirstPerThread(depthFirstArgs);

	pthread_mutex_destroy(&localLock);

	long long shrinkSize = sizeofItems + sizeofDats + sizeOfTailItems + sizeOfTailDats;
	shrinkSize += sizeof(ExtendedItemset) * 2;
	shrinkSize += sizeof(DepthFirstArgs);
	memoryManagers[threadNr]->shrink(shrinkSize);
}

void* FineGrainedParallelMiner::depthFirstPerThread(DepthFirstArgs* depthFirstArgs)
{
	long long shrinkSize = 0;
	vector<Thread*> helperThreads;

	int i = 0;
	while (true)
	{
		shrinkSize += tryAllocateNewThreads(depthFirstArgs, helperThreads);

		pthread_mutex_lock(depthFirstArgs->localLock);
		i = *depthFirstArgs->index;
		*depthFirstArgs->index += 1;
		pthread_mutex_unlock(depthFirstArgs->localLock);

		if (i < depthFirstArgs->tail->itemCount)
			depthFirstForItem(i, *depthFirstArgs->itemset, *depthFirstArgs->tail, *depthFirstArgs->newItemset, *depthFirstArgs->newTail, depthFirstArgs->threadNr);
		else
			break;
	}

	if (shrinkSize > 0)
	{
		waitWhileThreadsWorking(helperThreads);

		pthread_mutex_lock(&lock);
		for (int i = 0; i < helperThreads.size(); i++)
			availableThreads.push_back(&threads[helperThreads[i]->id]);
		newThreadsAvailable = true;
		pthread_mutex_unlock(&lock);

		memoryManagers[depthFirstArgs->threadNr]->shrink(shrinkSize);
	}

	return NULL;
}

long long FineGrainedParallelMiner::tryAllocateNewThreads(DepthFirstArgs* depthFirstArgs, vector<Thread*>& helperThreads)
{
	long long shrinkSize = 0;

	if (newThreadsAvailable && pthread_mutex_trylock(&lock) == 0)
	{
		if (availableThreads.size() > 0)
		{
			int dataCount = depthFirstArgs->tail->itemCount - *depthFirstArgs->index;
			int numThreads = getNumThreadsNeeded(availableThreads.size() + 1, dataCount); // + 1 in order to add current thread

			if (availableThreads.size() - numThreads + 1 <= 0)
				newThreadsAvailable = false;

			if (numThreads > 1)
			{
				int threadNr = depthFirstArgs->threadNr;
				ExtendedItemset* itemset = depthFirstArgs->itemset;
				ExtendedItemset* tail = depthFirstArgs->tail;

				ExtendedItemset* newItemset = (ExtendedItemset*)memoryManagers[threadNr]->grow(sizeof(ExtendedItemset) * numThreads);
				ExtendedItemset* newTail = (ExtendedItemset*)memoryManagers[threadNr]->grow(sizeof(ExtendedItemset) * numThreads);
				shrinkSize += sizeof(ExtendedItemset) * numThreads * 2;
				unsigned int itemsetItemCount = itemset->itemCount + 1;
				unsigned int tailItemCount = tail->itemCount - 1;
				long long sizeofItems = itemsetItemCount * sizeof(unsigned int*);
				long long sizeofDats = itemsetItemCount * sizeof(unsigned char*);
				long long sizeOfTailItems = tailItemCount * sizeof(unsigned int*);
				long long sizeOfTailDats = tailItemCount * sizeof(unsigned char*);

				for (int i = 0; i < numThreads - 1; i++)
				{
					newItemset[i].itemCount = itemsetItemCount;
					newItemset[i].items = (unsigned int*)memoryManagers[threadNr]->grow(sizeofItems);
					shrinkSize += sizeofItems;
					newItemset[i].dimensionalAttributes = memoryManagers[threadNr]->grow(sizeofDats);
					shrinkSize += sizeofDats;
					std::copy(itemset->items, itemset->items + itemset->itemCount, newItemset[i].items);
					std::copy(itemset->dimensionalAttributes, itemset->dimensionalAttributes + itemset->itemCount, newItemset[i].dimensionalAttributes);

					newTail[i].itemCount = tailItemCount;
					newTail[i].items = (unsigned int*)memoryManagers[threadNr]->grow(sizeOfTailItems);
					shrinkSize += sizeOfTailItems;
					newTail[i].dimensionalAttributes = memoryManagers[threadNr]->grow(sizeOfTailDats);
					shrinkSize += sizeOfTailDats;

					helperThreads.push_back(availableThreads.back());
					availableThreads.pop_back();

					DepthFirstArgs* newDepthFirstArgs = (DepthFirstArgs*)memoryManagers[threadNr]->grow(sizeof(DepthFirstArgs));
					shrinkSize += sizeof(DepthFirstArgs);

					newDepthFirstArgs->itemset = itemset;
					newDepthFirstArgs->tail = tail;
					newDepthFirstArgs->newItemset = &newItemset[i];
					newDepthFirstArgs->newTail = &newTail[i];
					newDepthFirstArgs->index = depthFirstArgs->index;
					newDepthFirstArgs->localLock = depthFirstArgs->localLock;
					newDepthFirstArgs->object = this;
					newDepthFirstArgs->threadNr = helperThreads.back()->id;

					helperThreads.back()->work = staticDepthFirstPerThread;
					helperThreads.back()->args = newDepthFirstArgs;
					helperThreads.back()->isWorking = true;
				}
			}
		}

		pthread_mutex_unlock(&lock);
	}

	return shrinkSize;
}

int FineGrainedParallelMiner::getNumThreadsNeeded(int numThreadsAvailable, int numItems)
{
	if (numThreadsAvailable >= numItems)
		return numItems;
	else
		return numThreadsAvailable;
}

void FineGrainedParallelMiner::checkIfAllThreadsFinished()
{
	bool allThreadsFinished = true;
	for (int i = 0; i < inputData.numThreads; i++)
	{
		if (!threads[i].finishedMainWork || threads[i].isWorking)
		{
			allThreadsFinished = false;
			break;
		}
	}
	if (allThreadsFinished)
		everythingFinished = true;
}

void FineGrainedParallelMiner::addAllSubsumeSets(unsigned int* items,
												 unsigned int itemCount,
												 unsigned int* subsume,
												 unsigned int subsumeCount,
												 unsigned char* dimensionalAttributes,
												 double support,
												 int threadNr)
{
	unsigned int power = 1 << (unsigned int)subsumeCount;
	int numThreads;
	vector<Thread*> helperThreads;
	bool isParallel = canBeParallelized(helperThreads, numThreads, power - 1);

	if (isParallel)
	{
		SubsumeCombinationsArgs* subsumeCombinationsArgs = (SubsumeCombinationsArgs*)memoryManagers[threadNr]->grow(sizeof(SubsumeCombinationsArgs) * helperThreads.size());

		for (int i = 0; i < helperThreads.size(); i++)
		{
			subsumeCombinationsArgs[i].start = i * power / numThreads;
			if (i == 0)
				subsumeCombinationsArgs[i].start++;
			subsumeCombinationsArgs[i].end = (i + 1) * power / numThreads;
			subsumeCombinationsArgs[i].items = items;
			subsumeCombinationsArgs[i].itemCount = itemCount;
			subsumeCombinationsArgs[i].subsume = subsume;
			subsumeCombinationsArgs[i].subsumeCount = subsumeCount;
			subsumeCombinationsArgs[i].dimensionalAttributes = dimensionalAttributes;
			subsumeCombinationsArgs[i].support = support;
			subsumeCombinationsArgs[i].threadNr = helperThreads[i]->id;
			subsumeCombinationsArgs[i].object = this;

			helperThreads[i]->work = staticAddAllSubsumeSetsPerThread;
			helperThreads[i]->args = &subsumeCombinationsArgs[i];
			helperThreads[i]->isWorking = true;
		}

		int start = helperThreads.size() * power / numThreads;
		addAllSubsumeSetsPerThread(start, power, items, itemCount, subsume, subsumeCount, dimensionalAttributes, support, threadNr);

		waitWhileThreadsWorking(helperThreads);

		pthread_mutex_lock(&lock);
		for (int i = 0; i < helperThreads.size(); i++)
			availableThreads.push_back(&threads[helperThreads[i]->id]);
		newThreadsAvailable = true;
		pthread_mutex_unlock(&lock);

		memoryManagers[threadNr]->shrink(sizeof(SubsumeCombinationsArgs) * helperThreads.size());
	}
	else
	{
		addAllSubsumeSetsPerThread(1, power, items, itemCount, subsume, subsumeCount, dimensionalAttributes, support, threadNr);
	}
}

void* FineGrainedParallelMiner::addAllSubsumeSetsPerThread(int start,
														   int end,
														   unsigned int* items,
														   unsigned int itemCount,
														   unsigned int* subsume,
														   unsigned int subsumeCount,
														   unsigned char* dimensionalAttributes,
														   double support,
														   int threadNr)
{
	ExtendedItemset output;
	output.itemCount = itemCount + subsumeCount;
	output.items = (unsigned int*)memoryManagers[threadNr]->grow(output.itemCount * sizeof(unsigned int));
	output.dimensionalAttributes = memoryManagers[threadNr]->grow(output.itemCount * sizeof(unsigned char));
	std::copy(items, items + itemCount, output.items);
	std::copy(dimensionalAttributes, dimensionalAttributes + itemCount, output.dimensionalAttributes);

	// Power set using counter
	for (unsigned int i = start; i < end; i++)
	{
		unsigned int bitCounter = i;

		int outputIndex = 0;
		for (int j = 0; j < subsumeCount; j++)
		{
			if (bitCounter & 1)
			{
				output.items[itemCount + outputIndex] = subsume[j];
				output.dimensionalAttributes[itemCount + outputIndex] = dimensionalAttributes[j + itemCount];
				outputIndex++;
			}
			bitCounter >>= 1;
		}
		output.itemCount = outputIndex + itemCount;

		if (support < inputData.minSup)
			continue;

		addToFrequentItemsets(output, support, threadNr);
	}

	memoryManagers[threadNr]->shrink(itemCount + subsumeCount);

	return NULL;
}

void FineGrainedParallelMiner::combineItemsSubsumeAndTail(unsigned int* items,
														  unsigned int itemCount,
														  unsigned int* subsume,
														  unsigned int subsumeCount,
														  unsigned char* dimensionalAttributes,
														  ExtendedItemset& tail,
														  int threadNr)
{
	unsigned int power = 1 << (unsigned int)subsumeCount;
	int numThreads;
	vector<Thread*> helperThreads;
	bool isParallel = canBeParallelized(helperThreads, numThreads, power - 1);

	if (isParallel)
	{
		SubsumeCombinationsArgs* subsumeCombinationsArgs = (SubsumeCombinationsArgs*)memoryManagers[threadNr]->grow(sizeof(SubsumeCombinationsArgs) * helperThreads.size());

		for (int i = 0; i < helperThreads.size(); i++)
		{
			subsumeCombinationsArgs[i].start = i * power / numThreads;
			if (i == 0)
				subsumeCombinationsArgs[i].start++;
			subsumeCombinationsArgs[i].end = (i + 1) * power / numThreads;
			subsumeCombinationsArgs[i].items = items;
			subsumeCombinationsArgs[i].itemCount = itemCount;
			subsumeCombinationsArgs[i].subsume = subsume;
			subsumeCombinationsArgs[i].subsumeCount = subsumeCount;
			subsumeCombinationsArgs[i].dimensionalAttributes = dimensionalAttributes;
			subsumeCombinationsArgs[i].tail = &tail;
			subsumeCombinationsArgs[i].threadNr = helperThreads[i]->id;
			subsumeCombinationsArgs[i].object = this;

			helperThreads[i]->work = staticCombineItemsSubsumeAndTailPerThread;
			helperThreads[i]->args = &subsumeCombinationsArgs[i];
			helperThreads[i]->isWorking = true;
		}

		int start = helperThreads.size() * power / numThreads;
		combineItemsSubsumeAndTailPerThread(start, power, items, itemCount, subsume, subsumeCount, dimensionalAttributes, &tail, threadNr);

		waitWhileThreadsWorking(helperThreads);

		pthread_mutex_lock(&lock);
		for (int i = 0; i < helperThreads.size(); i++)
			availableThreads.push_back(&threads[helperThreads[i]->id]);
		newThreadsAvailable = true;
		pthread_mutex_unlock(&lock);

		memoryManagers[threadNr]->shrink(sizeof(SubsumeCombinationsArgs) * helperThreads.size());
	}
	else
	{
		combineItemsSubsumeAndTailPerThread(1, power, items, itemCount, subsume, subsumeCount, dimensionalAttributes, &tail, threadNr);
	}
}

void* FineGrainedParallelMiner::combineItemsSubsumeAndTailPerThread(int start,
																	int end,
																	unsigned int* items,
																	unsigned int itemCount,
																	unsigned int* subsume,
																	unsigned int subsumeCount,
																	unsigned char* dimensionalAttributes,
																	ExtendedItemset* tail,
																	int threadNr)
{
	ExtendedItemset itemset;
	itemset.itemCount = itemCount + subsumeCount;
	itemset.items = (unsigned int*)memoryManagers[threadNr]->grow(itemset.itemCount * sizeof(unsigned int));
	itemset.dimensionalAttributes = memoryManagers[threadNr]->grow(itemset.itemCount * sizeof(unsigned char));
	std::copy(items, items + itemCount, itemset.items);
	std::copy(dimensionalAttributes, dimensionalAttributes + itemCount, itemset.dimensionalAttributes);

	// Power set using counter
	for (unsigned int i = start; i < end; i++)
	{
		unsigned int bitCounter = i;

		int itemsetIndex = 0;
		for (int j = 0; j < subsumeCount; j++)
		{
			if (bitCounter & 1)
			{
				itemset.items[itemCount + itemsetIndex] = subsume[j];
				itemset.dimensionalAttributes[itemCount + itemsetIndex] = dimensionalAttributes[j + itemCount];
				itemsetIndex++;
			}
			bitCounter >>= 1;
		}
		itemset.itemCount = itemsetIndex + itemCount;

		depthFirst(itemset, *tail, threadNr);
	}

	memoryManagers[threadNr]->shrink(itemCount + subsumeCount);

	return NULL;
}

bool FineGrainedParallelMiner::canBeParallelized(vector<Thread*>& helperThreads, int& numThreads, int dataCount)
{
	bool isParallel = false;
	numThreads = 1;
	if (newThreadsAvailable && pthread_mutex_trylock(&lock) == 0)
	{
		if (availableThreads.size() > 0)
		{
			numThreads = getNumThreadsNeeded(availableThreads.size() + 1, dataCount); // + 1 in order to add current thread
			if (numThreads > 1)
			{
				isParallel = true;

				for (int i = 0; i < numThreads - 1; i++)
				{
					helperThreads.push_back(availableThreads.back());
					availableThreads.pop_back();
				}

				if (availableThreads.size() == 0)
					newThreadsAvailable = false;
			}
		}

		pthread_mutex_unlock(&lock);
	}

	return isParallel;
}

void FineGrainedParallelMiner::waitWhileThreadsWorking(vector<Thread*>& threads)
{
	bool finished = false;
	while (!finished)
	{
		bool allThreadsFinished = true;
		for (int i = 0; i < threads.size(); i++)
		{
			if (threads[i]->isWorking)
			{
				allThreadsFinished = false;
				break;
			}
		}

		if (allThreadsFinished)
			finished = true;
	}
}
