#ifndef FINE_GRAINED_PARALLEL_MINER_H_
#define FINE_GRAINED_PARALLEL_MINER_H_

#include "Miner.h"

#include <vector>
using std::vector;
#include <pthread.h>

class FineGrainedParallelMiner;

struct Thread
{
	int id;
	pthread_t* thread;
	void* (*work)(void*);
	void* args;
	volatile bool isWorking;
	volatile bool finishedMainWork;
};

struct DepthFirstArgs
{
	FineGrainedParallelMiner* object;
	ExtendedItemset* itemset;
	ExtendedItemset* tail;
	ExtendedItemset* newItemset;
	ExtendedItemset* newTail;
	pthread_mutex_t* localLock;
	int* index;
	int threadNr;
};

class FineGrainedParallelMiner : public Miner
{
private:
	Thread* threads;
	vector<Thread*> availableThreads;
	volatile bool newThreadsAvailable;
	volatile bool everythingFinished;
	pthread_mutex_t lock;

	void indexBTFITI();
	void depthFirst(ExtendedItemset &itemset, ExtendedItemset &tail, int threadNr);

	int getNumThreadsNeeded(int numThreadsAvailable, int numItems);
	void checkIfAllThreadsFinished();

	void addAllSubsumeSets(unsigned int* items,
						   unsigned int itemCount,
						   unsigned int* subsume,
						   unsigned int subsumeCount,
						   unsigned char* dimensionalAttributes,
						   double support,
						   int threadNr);
	void combineItemsSubsumeAndTail(unsigned int* items,
									unsigned int itemCount,
									unsigned int* subsume,
									unsigned int subsumeCount,
									unsigned char* dimensionalAttributes,
									ExtendedItemset& tail,
									int threadNr);

	bool canBeParallelized(vector<Thread*>& helperThreads, int& numThreads, int dataCount);
	void waitWhileThreadsWorking(vector<Thread*>& threads);
public:
	FineGrainedParallelMiner(StackMemoryManager** memoryManagers,
							 DatasetPreprocessor& datasetPreprocessor,
							 BitTable& bitTable,
							 IndexArray& indexArray,
							 InputData& inputData,
							 LogData& logData);
	~FineGrainedParallelMiner();

	void* indexBTFITIPerThread(int threadNr, pthread_mutex_t* localLock, int* index);
	void* depthFirstPerThread(DepthFirstArgs *depthFirstArgs);
	long long tryAllocateNewThreads(DepthFirstArgs *depthFirstArgs, vector<Thread*>& helperThreads);

	void* addAllSubsumeSetsPerThread(int start,
									 int end,
									 unsigned int* items,
									 unsigned int itemCount,
									 unsigned int* subsume,
									 unsigned int subsumeCount,
									 unsigned char* dimensionalAttributes,
									 double support,
									 int threadNr);
	void* combineItemsSubsumeAndTailPerThread(int start,
											  int end,
											  unsigned int* items,
											  unsigned int itemCount,
											  unsigned int* subsume,
											  unsigned int subsumeCount,
											  unsigned char* dimensionalAttributes,
											  ExtendedItemset* tail,
											  int threadNr);
};

#endif // FINE_GRAINED_PARALLEL_MINER_H_
