#include "IndexBTFITI.h"
#include "FileLoader.h"
#include "DatasetPreprocessor.h"
#include "BitTable.h"
#include "StackMemoryManager.h"
#include "IndexArray.h"
#include "Miner.h"
#include "EmbarrassinglyParallelMiner.h"
#include "FineGrainedParallelMiner.h"

IndexBTFITI::IndexBTFITI(InputData& inputData, LogData& logData)
	: inputData(inputData), logData(logData)
{
}

void IndexBTFITI::startAlgorithm()
{
	MultiBuffer** multiBuffers = new MultiBuffer*[inputData.numThreads];
	FileLoader::loadFileToMultiBuffers(inputData.datasetPath, multiBuffers, inputData.numThreads);

	DatasetPreprocessor datasetPreprocessor(multiBuffers, inputData);

	StackMemoryManager** memoryManagers = new StackMemoryManager*[inputData.numThreads];
	for (int i = 0; i < inputData.numThreads; i++)
		memoryManagers[i] = new StackMemoryManager();

	BitTable bitTable(multiBuffers, inputData, datasetPreprocessor, memoryManagers, logData);
	for (int i = 0; i < inputData.numThreads; i++)
		delete multiBuffers[i];
	delete[] multiBuffers;

	IndexArray indexArray(memoryManagers, bitTable, inputData, datasetPreprocessor, logData);

	Miner* miner;
	if (inputData.miningMethod == MiningMethod::EMBARRASSINGLY_PARALLEL)
		miner = new EmbarrassinglyParallelMiner(memoryManagers, datasetPreprocessor, bitTable, indexArray, inputData, logData);
	else if (inputData.miningMethod == MiningMethod::FINE_GRAINED_PARALLEL)
		miner = new FineGrainedParallelMiner(memoryManagers, datasetPreprocessor, bitTable, indexArray, inputData, logData);
	miner->startMining();

	if (inputData.writeOutput)
		miner->writeResults(algorithmName + "_output.txt");
	else
		miner->reportNumberOfFrequentItemsetsFound();

	delete miner;
	for (int i = 0; i < inputData.numThreads; i++)
		delete memoryManagers[i];
	delete[] memoryManagers;
}
