#ifndef MINER_H_
#define MINER_H_

#include "StackMemoryManager.h"
#include "BitTable.h"
#include "IndexArray.h"
#include "ExtendedItemsetStack.h"
#include "DatasetPreprocessor.h"

struct ExtendedItemset
{
	unsigned int itemCount;
	unsigned int* items;
	unsigned char* dimensionalAttributes;
};

class Miner
{
protected:
	StackMemoryManager** memoryManagers;
	DatasetPreprocessor& datasetPreprocessor;
	ExtendedItemsetStack** extendedItemsetStacks;
	BitTable& bitTable;
	IndexArray& indexArray;
	InputData& inputData;
	LogData& logData;

	virtual void indexBTFITI() = 0;
	void indexBTFITIForItem(int i, IndexArrayElement* elements, int threadNr);
	void indexBitTableFI(unsigned int* items,
						 unsigned int itemCount,
						 unsigned int* subsume,
						 unsigned int subsumeCount,
						 unsigned char* dimensionalAttributes, int threadNr);
	virtual void depthFirst(ExtendedItemset& itemset, ExtendedItemset& tail, int threadNr) = 0;
	void depthFirstForItem(int i,
						   ExtendedItemset& itemset,
						   ExtendedItemset& tail,
						   ExtendedItemset& newItemset,
						   ExtendedItemset& newTail,
						   int threadNr);
	void addToFrequentItemsets(ExtendedItemset& extendedItemset, double support, int threadNr);
	double getSupport(ExtendedItemset& extendedItemset, int threadNr);
	unsigned int getSupportCount(ExtendedItemset& extendedItemset, int threadNr);
	unsigned int getHighestItemIdWithEqualOrGreaterSupport(unsigned int* items,
														   unsigned char* dats,
														   unsigned int itemCount,
														   int threadNr);
	virtual void addAllSubsumeSets(unsigned int* items,
						   unsigned int itemCount,
						   unsigned int* subsume,
						   unsigned int subsumeCount,
						   unsigned char* dimensionalAttributes,
						   double support,
						   int threadNr) = 0;
	virtual void combineItemsSubsumeAndTail(unsigned int* items,
									unsigned int itemCount,
									unsigned int* subsume,
									unsigned int subsumeCount,
									unsigned char* dimensionalAttributes,
									ExtendedItemset& tail,
									int threadNr) = 0;
public:
	Miner(StackMemoryManager** memoryManagers,
		  DatasetPreprocessor& datasetPreprocessor,
		  BitTable& bitTable,
		  IndexArray& indexArray,
		  InputData& inputData,
		  LogData& logData);
	virtual ~Miner();

	void startMining();
	void writeResults(string path);
	void reportNumberOfFrequentItemsetsFound();
};

#endif // MINER_H_
