#include <exception>
using std::exception;
#include <cstdlib>

#include "../FimLogger/Logger.h"
#include "StackMemoryManager.h"

StackMemoryManager::StackMemoryManager()
{
	currentBufferNr = 0;
	currentByteIndexInBuffer = 0;
	memoryBuffers = new unsigned char*[BUFFER_COUNT];
	memoryBuffers[0] = new unsigned char[MEMORY_BUFFER_SIZE];
	for (int i = 1; i < BUFFER_COUNT; i++)
		memoryBuffers[i] = NULL;
}

StackMemoryManager::~StackMemoryManager()
{
	for (int i = 0; i < BUFFER_COUNT; i++)
	{
		if (memoryBuffers[i] != NULL)
			delete[] memoryBuffers[i];
		else
			break;
	}
	delete[] memoryBuffers;
}

/***** PUBLIC FUNCTIONS *****/

unsigned char* StackMemoryManager::grow(long long byteCount)
{
	if (currentByteIndexInBuffer + byteCount > MEMORY_BUFFER_SIZE)
	{
		if (byteCount > MEMORY_BUFFER_SIZE)
		{
			writeToLog("Too many bytes. (StackMemoryManager::grow)");
			throw exception();
		}

		currentByteIndexInBuffer = 0;
		memoryBuffers[++currentBufferNr] = new unsigned char[MEMORY_BUFFER_SIZE];
	}

	unsigned char* memoryAddress = &memoryBuffers[currentBufferNr][currentByteIndexInBuffer];
	currentByteIndexInBuffer += byteCount;

	return memoryAddress;
}

void StackMemoryManager::shrink(long long byteCount)
{
	if (currentByteIndexInBuffer - byteCount < 0)
	{
		currentByteIndexInBuffer = MEMORY_BUFFER_SIZE + currentByteIndexInBuffer - byteCount;
		delete[] memoryBuffers[currentBufferNr];
		memoryBuffers[currentBufferNr] = NULL;
		currentBufferNr--;
	}
	else
		currentByteIndexInBuffer -= byteCount;
}

unsigned long long StackMemoryManager::getMemoryUsageInBytes() const
{
	return MEMORY_BUFFER_SIZE * (currentBufferNr) + currentByteIndexInBuffer;
}

/***** PRIVATE FUNCTIONS *****/
