#include "EquivalenceClassStack.h"

#include <cstdlib>

EquivalenceClassStack::EquivalenceClassStack(unsigned int numBitmapBytes)
	: NUM_BITMAP_BYTES(numBitmapBytes)
{
	equivalenceClassCount = 0;
	topItemCountAddress = NULL;
}

/***** PUBLIC FUNCTIONS *****/

void EquivalenceClassStack::push(unsigned int itemCount, unsigned int* item, unsigned char* tidBitmap)
{
	unsigned long long sizeofItems = itemCount * sizeof(unsigned int);
	unsigned long long sizeofBitmap = NUM_BITMAP_BYTES * sizeof(unsigned char);
	unsigned long long sizeofItemCount = sizeof(unsigned int);

	unsigned long long memoryChunkSize = sizeofItems + sizeofBitmap + sizeofItemCount;;
	unsigned char* itemAddress = memoryManager.grow(memoryChunkSize);
	for (unsigned int i = 0; i < itemCount; i++)
		itemAddress[i] = item[i];

	unsigned char* bitmapAddress = (unsigned char*)(itemAddress + sizeofItems);
	for (unsigned long long i = 0; i < NUM_BITMAP_BYTES; i++)
		bitmapAddress[i] = tidBitmap[i];

	topItemCountAddress = (unsigned int*)(itemAddress + sizeofItems + sizeofBitmap);
	*topItemCountAddress = itemCount;

	equivalenceClassCount++;
}

/*
The itemAddr and tidBitmapAddr variables must be set outside this function.
*/
void EquivalenceClassStack::pushEmpty(unsigned int itemCount,
									  unsigned int** itemAddress,
									  unsigned char** tidBitmapAddress)
{
	unsigned long long sizeofItems = itemCount * sizeof(unsigned int);
	unsigned long long sizeofBitmap = NUM_BITMAP_BYTES * sizeof(unsigned char);
	unsigned long long sizeofItemCount = sizeof(unsigned int);

	unsigned long long memoryChunkSize = sizeofItems + sizeofBitmap + sizeofItemCount;;
	unsigned char* startAddress = memoryManager.grow(memoryChunkSize);
	*itemAddress = (unsigned int*)startAddress;
	*tidBitmapAddress = (unsigned char*)(startAddress + sizeofItems);

	topItemCountAddress = (unsigned int*)(startAddress + sizeofItems + sizeofBitmap);
	*topItemCountAddress = itemCount;

	equivalenceClassCount++;
}

void EquivalenceClassStack::top(unsigned int** itemCount, unsigned int** items, unsigned char** tidBitmap) const
{
	unsigned long long sizeofItems = (*topItemCountAddress) * sizeof(unsigned int);
	unsigned long long sizeofBitmap = NUM_BITMAP_BYTES * sizeof(unsigned char);

	*itemCount = topItemCountAddress;
	*tidBitmap = (unsigned char*)((unsigned char*)topItemCountAddress - sizeofBitmap);
	*items = (unsigned int*)((unsigned char*)topItemCountAddress - sizeofBitmap - sizeofItems);
}

void EquivalenceClassStack::pop()
{
	unsigned int itemCount = *topItemCountAddress;
	unsigned long long sizeofItems = itemCount * sizeof(unsigned int);
	unsigned long long sizeofBitmap = NUM_BITMAP_BYTES * sizeof(unsigned char);
	unsigned long long sizeofItemCount = sizeof(unsigned int);

	unsigned long long shrinkAmount = sizeofItems + sizeofBitmap + sizeofItemCount;
	memoryManager.shrink(shrinkAmount);
	topItemCountAddress = (unsigned int*)((unsigned char*)topItemCountAddress - shrinkAmount);

	equivalenceClassCount--;
}

int EquivalenceClassStack::size() const
{
	return equivalenceClassCount;
}

unsigned long long EquivalenceClassStack::getMemoryUsageInBytes() const
{
	return memoryManager.getMemoryUsageInBytes();
}
