#include "McEclat.h"
#include "FileLoader.h"
#include "DatasetPreprocessor.h"
#include "VerticalDataLayout.h"
#include "../FimLogger/Logger.h"
#include "Utility.h"
#include "EclatMiner.h"
#include "IndependentClassMiner.h"
#include "HybridClassMiner.h"

#include <iostream>
using std::cout;
using std::endl;

McEclat::McEclat(InputData &inputData, LogData &logData)
	: inputData(inputData), logData(logData)
{
}

void McEclat::startAlgorithm()
{
	MultiBuffer** multiBuffers = new MultiBuffer*[inputData.numThreads];
	FileLoader::loadFileToMultiBuffers(inputData.datasetPath, multiBuffers, inputData.numThreads);
	DatasetPreprocessor datasetPreprocessor(multiBuffers, inputData.minSup, inputData.numThreads);
	VerticalDataLayout verticalDataLayout(datasetPreprocessor, multiBuffers, logData);

	EclatMiner* miner = NULL;
	if (inputData.miningMethod == MiningMethod::INDEPENDENT_CLASS)
		miner = new IndependentClassMiner(verticalDataLayout, datasetPreprocessor, logData);
	else if (inputData.miningMethod == MiningMethod::HYBRID)
		miner = new HybridClassMiner(verticalDataLayout, datasetPreprocessor, logData);
	miner->startMining();

	if (inputData.writeOutput)
		miner->writeResults(algorithmName + "_output.txt");
	else
		miner->reportNumberOfFrequentItemsetsFound();

	delete miner;
}
