#ifndef VERTICAL_DATA_LAYOUT_H_
#define VERTICAL_DATA_LAYOUT_H_

#include "DatasetPreprocessor.h"
#include "MultiBuffer.h"
#include "StackMemoryManager.h"
#include "../FimLogger/Logger.h"

#include <vector>
using std::vector;

class VerticalDataLayout
{
private:
	DatasetPreprocessor& datasetPreprocessor;
	MultiBuffer** multiBuffers;
	LogData& logData;
	StackMemoryManager** stackMemoryManager;
	int numThreads;
	unsigned long long numRows;
	unsigned long long numColumns;
	unsigned char** tidBitmaps;
	vector<unsigned int> frequentItems;

	void setFrequentItems();
	void allocateMemory();
	void createVerticalDataLayout();
	int getFirstTransactionNr(int threadNr);
	int getLastTransactionNr(int threadNr);
public:
	VerticalDataLayout(DatasetPreprocessor& datasetPreprocessor,
					   MultiBuffer** multiBuffers,
					   LogData& logData);
	~VerticalDataLayout();

	vector<unsigned int>* getFrequentItems();
	unsigned long long getNumBitsPerBitmap();
	unsigned char** getTidBitmaps() const;
	unsigned long long getMemoryUsageInBytes() const;
};

#endif // VERTICAL_DATA_LAYOUT_H_
