#include "McEclat.h"
#include "Utility.h"
#include "../FimLogger/Logger.h"
#include "../FimLogger/Logger.cpp"

#include <iostream>
using std::cout;
using std::endl;
#include <string>
using std::string;
#include <cstring>
#include <omp.h>

InputData inputData;
string algorithmName = "ParallelEclat"; // Used when writing to log in Logger.cpp

void writeInputDataToLog()
{
	writeToLog("Number of threads: " + std::to_string(inputData.numThreads) + "\n");
	writeToLog("Minimum support: " + std::to_string(inputData.minSup) + "\n");
	writeToLog("Dataset path: " + inputData.datasetPath + "\n");

	string writeOutput = inputData.writeOutput ? "yes" : "no";
	writeToLog("Write output: " + writeOutput + "\n");

	string miningMethod = "";
	switch (inputData.miningMethod) {
	case MiningMethod::INDEPENDENT_CLASS:
		miningMethod = "INDEPENDENT_CLASS";
		break;
	case MiningMethod::HYBRID:
		miningMethod = "HYBRID";
	default:
		break;
	}
	writeToLog("Mining method: " + miningMethod + "\n\n");
}

bool readInput(int argc, char** argv)
{
	if (argc == 1)
	{
		cout << "Input format" << endl;
		cout << "-d <dataset path> \t\t\t\t\t(default: \"" << inputData.datasetPath << "\")" << endl;
		cout << "-s <minimum support (value in range [0.0,1.0])> \t(default: " << inputData.minSup << ")" << endl;
		cout << "-t <number of threads> \t\t\t\t\t(default: " << inputData.numThreads << ")" << endl;
		cout << "-w <write output (0: no, 1: yes)> \t\t\t(default: " << (inputData.writeOutput ? 1 : 0) << ")" << endl;
		cout << "-m <mining method (0: IndependentClass, 1: Hybrid)> \t(default: " << (int)inputData.miningMethod << ")" << endl;
		cout << "-i <input file> \t\t\t\t\t(default: \"" << inputData.inputFile << "\")" << endl;
		cout << "-l <log file path> \t\t\t\t\t(default: \"" << inputData.logFilePath << "\")" << endl;
		cout << "-r <repetitionNr> \t\t\t\t\t(default: \"" << inputData.repetition << "\")" << endl;
		return false;
	}

	int argnr = 0;
	while (++argnr < argc)
	{
		if (!strcmp(argv[argnr], "-d"))
			inputData.datasetPath = argv[++argnr];
		else if (!strcmp(argv[argnr], "-s"))
			inputData.minSup = atof(argv[++argnr]);
		else if (!strcmp(argv[argnr], "-t"))
			inputData.numThreads = atoi(argv[++argnr]);
		else if (!strcmp(argv[argnr], "-w"))
			inputData.writeOutput = atoi(argv[++argnr]) != 0;
		else if (!strcmp(argv[argnr], "-m"))
			inputData.miningMethod = static_cast<MiningMethod>(atoi(argv[++argnr]));
		else if (!strcmp(argv[argnr], "-i"))
			inputData.inputFile = argv[++argnr];
		else if (!strcmp(argv[argnr], "-l"))
			inputData.logFilePath = argv[++argnr];
		else if (!strcmp(argv[argnr], "-r"))
			inputData.repetition = atoi(argv[++argnr]);
	}

	return true;
}

void runAlgorithm(LogData* logData)
{
	double total_time = omp_get_wtime();
	writeToLog("\n------------------------------------------------------------------------\n\n");
	writeToLog("Time run: " + currentDateTime() + "\n\n");
	writeInputDataToLog();

	McEclat mcEclat(inputData, *logData);
	mcEclat.startAlgorithm();

	writeToLog("Total time: " + std::to_string(omp_get_wtime() - total_time) + "s\n\n");
	writeToLog("------------------------------------------------------------------------\n\n");

	if (inputData.repetition != -1) // Is run from script hence write statistics
	{
		logData->repetitionNr = inputData.repetition;
		logData->support = inputData.minSup;
		logData->threads = inputData.numThreads;
		logData->dataset = inputData.datasetPath;
		logData->algorithmName = algorithmName;
		Logger::writeLogData(*logData, inputData.logFilePath);
	}
}

int main(int argc, char** argv)
{
	if (readInput(argc, argv))
	{
		if (!inputData.inputFile.empty())
		{
			Logger log(inputData.inputFile, inputData.logFilePath);
			int testNr = 1;
			int numTests = log.getNumRepetitions();
			numTests *= log.getSupports()->size();
			numTests *= log.getThreads()->size();
			numTests *= log.getDatasets()->size();

			for (size_t repetition = 0; repetition < log.getNumRepetitions(); repetition++)
			{
				for (size_t supportIndex = 0; supportIndex < log.getSupports()->size(); supportIndex++)
				{
					for (size_t threadIndex = 0; threadIndex < log.getThreads()->size(); threadIndex++)
					{
						for (size_t datasetIndex = 0; datasetIndex < log.getDatasets()->size(); datasetIndex++)
						{
							inputData.datasetPath = log.getDatasets()->at(datasetIndex);
							inputData.numThreads = log.getThreads()->at(threadIndex);
							inputData.minSup = log.getSupports()->at(supportIndex);

							LogData logData;
							logData.repetitionNr = repetition;
							logData.supportIndex = supportIndex;
							logData.threadIndex = threadIndex;
							logData.datasetIndex = datasetIndex;
							logData.algorithmName = algorithmName;

							writeToLog("Test: " + toString(testNr++) + "/" + toString(numTests));
							runAlgorithm(&logData);

							log.writeLogData(logData);
						}
					}
				}
			}
		}
		else
		{
			LogData logData;
			runAlgorithm(&logData);
		}
	}

	return 0;
}
