#ifndef CUSTOM_VECTOR_H_
#define CUSTOM_VECTOR_H_

#include "StackMemoryManager.h"

#include <cstring>

template <typename T>
class CustomVector
{
private:
	StackMemoryManager* memoryManager;
	T* array;
	int lastIndex;
	unsigned int arraySize;
public:
	CustomVector(StackMemoryManager* memoryManager, unsigned int initialSize = 128)
	{
		constructor(memoryManager, initialSize);
	}

	void constructor(StackMemoryManager* memoryManager, unsigned int initialSize = 128)
	{
		this->memoryManager = memoryManager;
		this->arraySize = initialSize;
		array = (T*)memoryManager->grow(arraySize * sizeof(T));
		lastIndex = 0;
	}

	~CustomVector()
	{
		// WARNING: This method does not delete the memory that was allocated.
	}

	T& operator[](const int index)
	{
		return array[index];
	}

	T& at(const int index)
	{
		return array[index];
	}

	T& back()
	{
		return array[lastIndex - 1];
	}

	void push_back(T value)
	{
		lastIndex++;
		if (lastIndex == arraySize)
		{
			arraySize *= 2;
			T* newArray = (T*)memoryManager->grow(arraySize * sizeof(T));
			memcpy(newArray, array, lastIndex * sizeof(T));
			array = newArray;
		}
		array[lastIndex - 1] = value;
	}

	unsigned int size()
	{
		return lastIndex;
	}

	bool empty()
	{
		return lastIndex == 0;
	}

	void clear()
	{
		lastIndex = 0;
	}
};

#endif // CUSTOM_VECTOR_H_
