#include "FileLoader.h"
#include "../FimLogger/Logger.h"

#include <fstream>
using std::ifstream;
using std::exception;
#include <omp.h>

/*
Reads the specified file from disk into memory.
Divides the file into multiple buffers, the number is given by numBuffers.
*/
void FileLoader::loadFileToBuffers(string path, MultiBuffer*** multiBuffers, int numBuffers)
{
	ifstream file(path.c_str());
	if (file.is_open())
	{
		string startMessage = "Reading file: " + path + "\n";
		writeToLog(startMessage);
		double time = omp_get_wtime();

		file.seekg(0, file.end);
		int fileLength = (int)file.tellg();
		file.seekg(0, file.beg);

		// Create file partitions for each thread
		int* partitions = new int[numBuffers + 1];
		partitions[0] = 0;
		int approxPartitionSize = fileLength / numBuffers;
		for (int i = 0; i < numBuffers; i++)
		{
			partitions[i + 1] = partitions[i] + approxPartitionSize;
			if (partitions[i + 1] > fileLength)
				partitions[i + 1] = fileLength;
			if (partitions[i + 1] == fileLength)
				continue; // continue so the next partitions can be set to fileLength as well

			// find new line character
			file.seekg(partitions[i + 1]);
			char c;
			do
			{
				file.get(c);
				partitions[i + 1] += 1;
			} while (c != '\n');
		}
		file.seekg(0, file.beg);

		for (int k = 0; k < numBuffers; k++)
		{
			int partitionSize = partitions[k + 1] - partitions[k];
			(*multiBuffers)[k] = new MultiBuffer(partitionSize);
			int filePosition = partitions[k];
			for (int i = 0; i < (*multiBuffers)[k]->getNumBuffers(); i++)
			{
				int numBytes;
				// determine buffer size
				if (partitionSize <= (*multiBuffers)[k]->BUFFER_SIZE)
					numBytes = partitionSize;
				else if (i != (*multiBuffers)[k]->getNumBuffers() - 1)
					numBytes = (*multiBuffers)[k]->BUFFER_SIZE;
				else
					numBytes = partitionSize - (*multiBuffers)[k]->BUFFER_SIZE * ((*multiBuffers)[k]->getNumBuffers() - 1);

				// read part of the file into buffer
				char* b = new char[numBytes];
				file.read(b, numBytes);
				filePosition += numBytes;
				file.seekg(filePosition);
				// add the buffer into the multibuffer
				(*multiBuffers)[k]->addBuffer((byte*)b);
			}
		}
		delete[] partitions;

		string endMessage = "File read into memory. [" + toString(omp_get_wtime() - time) + "s]\n\n";
		writeToLog(endMessage);
	}
	else
	{
		writeToLog("File not found.\n\n");
		throw exception();
	}
}
