#include "MultiBuffer.h"

#include <cmath>

#ifndef NULL
#define NULL 0
#endif

MultiBuffer::MultiBuffer(int numBytes)
	: numBytes(numBytes),
	  numBuffersAdded(0)
{
	numBuffers = (int)ceil(numBytes / (double)BUFFER_SIZE);
	buffers = new byte*[numBuffers];

	// initialize buffers to NULL
	for (int i = 0; i < numBuffers; i++)
		buffers[i] = NULL;
}

MultiBuffer::~MultiBuffer()
{
	for (int i = 0; i < numBuffers; i++)
		removeBuffer(&i);

	delete[] buffers;
}

/***** PUBLIC FUNCTIONS *****/

void MultiBuffer::addBuffer(byte* buffer)
{
	buffers[numBuffersAdded++] = buffer;
}

byte MultiBuffer::getByte(const int* byteNr)
{
	int bufferIndex = getBufferIndex(byteNr);
	int byteIndex = getByteIndex(byteNr);
	return buffers[bufferIndex][byteIndex];
}

/*
Reads byte and deletes the buffer if the byte was the last in that buffer.
*/
byte MultiBuffer::getByteAndDelete(const int* byteNr)
{
	int bufferIndex = getBufferIndex(byteNr);
	int byteIndex = getByteIndex(byteNr);

	if (byteIndex == BUFFER_SIZE - 1 || *byteNr == numBytes - 1)
	{
		char byte = buffers[bufferIndex][byteIndex];
		removeBuffer(&bufferIndex);
		return byte;
	}

	return buffers[bufferIndex][byteIndex];
}

int MultiBuffer::getNumBuffers()
{
	return numBuffers;
}

int MultiBuffer::getNumBytes()
{
	return numBytes;
}

/***** PRIVATE FUNCTIONS *****/

int MultiBuffer::getBufferIndex(const int* byteNr)
{
	return *byteNr / BUFFER_SIZE;
}

int MultiBuffer::getByteIndex(const int* byteNr)
{
	return *byteNr - getBufferIndex(byteNr) * BUFFER_SIZE;
}

void MultiBuffer::removeBuffer(const int* bufferIndex)
{
	if (buffers[*bufferIndex] != NULL)
	{
		delete[] buffers[*bufferIndex];
		buffers[*bufferIndex] = NULL;
	}
}
