#include "XfpTree.h"
#include "PreProcessor.h"

#include <omp.h>

XfpTree::XfpTree(FPTree** fpTrees, int numThreads) : NUM_THREADS(numThreads)
{
	mergeFpTrees(fpTrees);
}

XfpTree::~XfpTree()
{
	delete[] headerTable.itemCounts;
	delete[] headerTable.nodes;
}

/***** PUBLIC FUNCTIONS *****/

HeaderTable* XfpTree::getHeaderTable()
{
	return &headerTable;
}

/***** PRIVATE FUNCTIONS *****/

/*
Merges the fp-trees into an xfp-tree
*/
void XfpTree::mergeFpTrees(FPTree** fpTrees)
{
	headerTable.numItems = PreProcessor::getCountListBufferSize();
	headerTable.itemCounts = new int[headerTable.numItems];
	headerTable.nodes = new FPNode*[headerTable.numItems];

	int partitionSize = headerTable.numItems / NUM_THREADS;

	#pragma omp parallel num_threads(NUM_THREADS)
	{
		int threadNr = omp_get_thread_num();

		int start = threadNr * partitionSize;
		int end = threadNr == NUM_THREADS - 1 ? headerTable.numItems : (threadNr + 1) * partitionSize;

		for (int i = start; i < end; i++)
		{
			headerTable.itemCounts[i] = PreProcessor::getGlobalCountList()[PreProcessor::getRemapToInitialIdArray()[i]];

			FPNode* currentNode = NULL;
			for (int j = 0; j < NUM_THREADS; j++)
			{
				if (currentNode == NULL)
				{
					currentNode = fpTrees[j]->getHeaderTable()->nodes[i];
					headerTable.nodes[i] = currentNode; // First node in header table
					continue;
				}

				while (currentNode->nextEqualItem != NULL)
					currentNode = currentNode->nextEqualItem;

				currentNode->nextEqualItem = fpTrees[j]->getHeaderTable()->nodes[i];
			}
		}
	}
}
