#ifndef XFP_TREE_MINER_H_
#define XFP_TREE_MINER_H_

#include "XfpTree.h"
#include "ItemsetStack.h"
#include "CustomVector.h"
#include "HashTable.h"

#include <string>
using std::string;
#include <vector>
using std::vector;
#include <set>

struct Itemset
{
	unsigned int* items;
	unsigned int itemCount;
	double support;
};

struct BitsetVector
{
	Bitset* bitsets;
	int itemCount;
	int* items;
};

struct CalculateSupportCountContext
{
	// For MineBitVector
	BitsetVector* bitsetVector;
	CustomVector<Bitset>* bitsets;
	int* weights;
	int itemCount;
	unsigned int bitCounter;

	// For MineFPTree
	HashTable<HashHeaderTableElement*>* hashHeaderTable;
	int currentItem;
};

class XfpTreeMiner
{
	private:
		const int NUM_THREADS;
		double minSup;
		int numTransactions;
		int minNumTransactions;
		XfpTree* xfpTree;
		ItemsetStack** itemsetStacks;
		StackMemoryManager** memoryManagers;

		const int STEP;
		const int N;
		int* K;
		int** X;

		LogData& logData;

		void ParallelMinePattern();
		void MineFPTree(FPTree* tree, int* suffix, int suffixSize);
		void StartMineBitVector(FPNode* node, int sizeOfConditionalPatternBase);
		void MineBitVector(BitsetVector* bitsetVector, int* weights, int* suffix, int suffixSize);
		void UpdateK(int numNewPatters, int size, int threadNr);

		bool addToFrequentItemsets(int threadNr, int* items, unsigned int itemCount, int supportCount);
		int getSizeOfConditionalPatternBase(FPNode* bottomNode, int* numUniqueItems);
		void joinTwoArrays(int* array1, int sizeArray1, int* array2, int sizeArray2, int* destination);
		int getSupport(Bitset* bitset, int* weights);
		void putAllItemsFromConditionalPatternBaseToSet(HashTable<int> *set, FPNode* node);
		void initializeBitsets(BitsetVector* bitsetVector, FPNode* node);
		void setWeights(int* weights, FPNode* node);
		FPTree* constructConditionalFpTree(FPNode* firstNode, FPTree* fpTree);

		void addRestOfItemsToFrequentItemsets(CustomVector<int> *items,
											  int* currentItemset,
											  int currentItemsetSize,
											  int (XfpTreeMiner::*calcSupport)(CalculateSupportCountContext*),
											  CalculateSupportCountContext* context);
		int calcuateSupportForBitsetVector(CalculateSupportCountContext* context);
		int calculateSupportForFpTree(CalculateSupportCountContext* context);
		int calculateSupportForFPTreeNonSinglePath(FPNode* bottomNode, int forItem);
	public:
		XfpTreeMiner(XfpTree* xfpTree,
					 int numThreads,
					 double minSup,
					 int numTransactions,
					 LogData& logData);
		~XfpTreeMiner();

		void startMiningProcedure();
		void writeResults(string path);
		void reportNumberOfFrequentItemsetsFound();
};

#endif /* XFP_TREE_MINER_H_ */
